package javax.swing;

import javax.accessibility.AccessibleTextSequence;
import java.util.Arrays;
import javax.accessibility.AccessibleText;
import javax.accessibility.AccessibleRole;
import javax.accessibility.AccessibleContext;
import java.io.IOException;
import java.io.ObjectOutputStream;
import javax.swing.text.Segment;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;

public class JPasswordField extends JTextField
{
    private static final String uiClassID = "PasswordFieldUI";
    private char echoChar;
    private boolean echoCharSet;
    
    public JPasswordField() {
        this(null, null, 0);
    }
    
    public JPasswordField(String s) {
        this(null, s, 0);
    }
    
    public JPasswordField(int n) {
        this(null, null, n);
    }
    
    public JPasswordField(String s, int n) {
        this(null, s, n);
    }
    
    public JPasswordField(Document document, String s, int n) {
        super(document, s, n);
        this.enableInputMethods(this.echoCharSet = false);
    }
    
    @Override
    public String getUIClassID() {
        return "PasswordFieldUI";
    }
    
    @Override
    public void updateUI() {
        if (!this.echoCharSet) {
            this.echoChar = '*';
        }
        super.updateUI();
    }
    
    public char getEchoChar() {
        return this.echoChar;
    }
    
    public void setEchoChar(char echoChar) {
        this.echoChar = echoChar;
        this.echoCharSet = true;
        this.repaint();
        this.revalidate();
    }
    
    public boolean echoCharIsSet() {
        return this.echoChar != '\0';
    }
    
    @Override
    public void cut() {
        if (this.getClientProperty("JPasswordField.cutCopyAllowed") != Boolean.TRUE) {
            UIManager.getLookAndFeel().provideErrorFeedback(this);
        }
        else {
            super.cut();
        }
    }
    
    @Override
    public void copy() {
        if (this.getClientProperty("JPasswordField.cutCopyAllowed") != Boolean.TRUE) {
            UIManager.getLookAndFeel().provideErrorFeedback(this);
        }
        else {
            super.copy();
        }
    }
    
    @Deprecated
    @Override
    public String getText() {
        return super.getText();
    }
    
    @Deprecated
    @Override
    public String getText(int offs, int len) throws BadLocationException {
        return super.getText(offs, len);
    }
    
    public char[] getPassword() {
        Document document = this.getDocument();
        Segment segment = new Segment();
        try {
            document.getText(0, document.getLength(), segment);
        }
        catch (BadLocationException ex) {
            return null;
        }
        char[] array = new char[segment.count];
        System.arraycopy(segment.array, segment.offset, array, 0, segment.count);
        return array;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        if (this.getUIClassID().equals("PasswordFieldUI")) {
            int n = (byte)(JComponent.getWriteObjCounter(this) - 1);
            JComponent.setWriteObjCounter(this, (byte)n);
            if (n == 0 && this.ui != null) {
                this.ui.installUI(this);
            }
        }
    }
    
    @Override
    protected String paramString() {
        return super.paramString() + ",echoChar=" + this.echoChar;
    }
    
    boolean customSetUIProperty(String s, Object o) {
        if (s == "echoChar") {
            if (!this.echoCharSet) {
                this.setEchoChar((char)o);
                this.echoCharSet = false;
            }
            return true;
        }
        else {
            return false;
        }
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new JPasswordField.AccessibleJPasswordField();
        }
        return this.accessibleContext;
    }
    
    protected class AccessibleJPasswordField extends JTextField.AccessibleJTextField
    {
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.PASSWORD_TEXT;
        }
        
        @Override
        public AccessibleText getAccessibleText() {
            return this;
        }
        
        private String getEchoString(String s) {
            if (s == null) {
                return null;
            }
            char[] array = new char[s.length()];
            Arrays.fill(array, JPasswordField.this.getEchoChar());
            return new String(array);
        }
        
        @Override
        public String getAtIndex(int part, int index) {
            String atIndex;
            if (part == 1) {
                atIndex = super.getAtIndex(part, index);
            }
            else {
                char[] password = JPasswordField.this.getPassword();
                if (password == null || index < 0 || index >= password.length) {
                    return null;
                }
                atIndex = new String(password);
            }
            return this.getEchoString(atIndex);
        }
        
        @Override
        public String getAfterIndex(int part, int index) {
            if (part == 1) {
                return this.getEchoString(super.getAfterIndex(part, index));
            }
            return null;
        }
        
        @Override
        public String getBeforeIndex(int part, int index) {
            if (part == 1) {
                return this.getEchoString(super.getBeforeIndex(part, index));
            }
            return null;
        }
        
        @Override
        public String getTextRange(int startIndex, int endIndex) {
            return this.getEchoString(super.getTextRange(startIndex, endIndex));
        }
        
        @Override
        public AccessibleTextSequence getTextSequenceAt(int part, int index) {
            if (part == 1) {
                AccessibleTextSequence textSequence = super.getTextSequenceAt(part, index);
                if (textSequence == null) {
                    return null;
                }
                return new AccessibleTextSequence(textSequence.startIndex, textSequence.endIndex, this.getEchoString(textSequence.text));
            }
            else {
                char[] password = JPasswordField.this.getPassword();
                if (password == null || index < 0 || index >= password.length) {
                    return null;
                }
                return new AccessibleTextSequence(0, password.length - 1, this.getEchoString(new String(password)));
            }
        }
        
        @Override
        public AccessibleTextSequence getTextSequenceAfter(int part, int index) {
            if (part == 1) {
                AccessibleTextSequence textSequenceAfter = super.getTextSequenceAfter(part, index);
                if (textSequenceAfter == null) {
                    return null;
                }
                return new AccessibleTextSequence(textSequenceAfter.startIndex, textSequenceAfter.endIndex, this.getEchoString(textSequenceAfter.text));
            }
            else {
                return null;
            }
        }
        
        @Override
        public AccessibleTextSequence getTextSequenceBefore(int part, int index) {
            if (part == 1) {
                AccessibleTextSequence textSequenceBefore = super.getTextSequenceBefore(part, index);
                if (textSequenceBefore == null) {
                    return null;
                }
                return new AccessibleTextSequence(textSequenceBefore.startIndex, textSequenceBefore.endIndex, this.getEchoString(textSequenceBefore.text));
            }
            else {
                return null;
            }
        }
    }
}
